@@ -1,13 +1,17 @@
 mod common;
+mod exceptions;
+mod sim;
 
 use crate::common::DHeader;
 use crate::common::DImage;
 use crate::common::DetectorConfig;
-use crate::common::FrameSender;
+use crate::sim::FrameSender;
 use serde::Serialize;
 use std::collections::HashMap;
 use std::io;
 use std::io::Write;
+use std::time::Instant;
+use zmq::Message;
 
 use crate::common::DumpRecordFile;
 use clap::{Parser, Subcommand};
@@ -44,6 +48,9 @@ enum Action {
     Sim {
         uri: String,
     },
+    Record {
+        uri: String,
+    },
 }
 
 fn action_cat(cli: &Cli, start_idx: usize, end_idx: usize) {
@@ -161,9 +168,16 @@ fn action_inspect(cli: &Cli, head: Option<usize>, summary: bool) {
 }
 
 fn write_raw_msg(msg: &[u8]) {
+    write_raw_msg_fh(msg, &mut io::stdout());
+}
+
+fn write_raw_msg_fh<W>(msg: &[u8], out: &mut W)
+where
+    W: Write,
+{
     let length = (msg.len() as i64).to_le_bytes();
-    io::stdout().write_all(&length).unwrap();
-    io::stdout().write_all(msg).unwrap();
+    out.write_all(&length).unwrap();
+    out.write_all(msg).unwrap();
 }
 
 fn write_serializable<T>(value: &T)
@@ -190,7 +204,7 @@ fn action_repeat(cli: &Cli, repetitions: usize) {
     let mut detector_config_value: serde_json::Value =
         serde_json::from_slice::<serde_json::Value>(detector_config_msg).unwrap();
 
-    // XXX the heaer may lie about the number of images:
+    // XXX the header may lie about the number of images:
     let summary = get_summary(&cli.filename);
     let nimages = summary.get("<binary>").unwrap();
     let dest_num_images = nimages * repetitions;
@@ -244,13 +258,90 @@ fn action_sim(filename: &str, uri: &str) {
     sender.send_footer();
 }
 
+fn action_record(filename: &String, uri: &String) {
+    let raw_file = std::fs::File::create(filename).unwrap();
+    let mut file = std::io::BufWriter::new(raw_file);
+    let ctx = zmq::Context::new();
+    let socket = ctx.socket(zmq::PULL).unwrap();
+
+    let mut size: usize = 0;
+
+    socket.connect(uri).unwrap();
+    socket.set_rcvhwm(4 * 2048).unwrap();
+
+    let mut first_header: Message = Message::new();
+
+    println!("synchronizing to first header");
+    // synchronize to first header:
+    let dheader = loop {
+        // First acquisition header: Protocol etc
+        socket.recv(&mut first_header, 0).unwrap();
+        let first_header = first_header.as_str();
+        if let Some(first_header) = first_header {
+            let dheader_res: Result<DHeader, _> = serde_json::from_str(first_header);
+            if dheader_res.is_ok() {
+                break dheader_res.unwrap();
+            }
+        }
+    };
+
+    println!("got first header: {dheader:?}");
+
+    let start = Instant::now();
+
+    socket.set_rcvtimeo(1000).unwrap();
+
+    let mut detector_config_msg: Message = Message::new();
+    // Second acquisition header: Detector config
+    socket.recv(&mut detector_config_msg, 0).unwrap();
+
+    let detector_config: DetectorConfig = serde_json::from_slice(&detector_config_msg).unwrap();
+
+    println!("detector config: {:?}", detector_config);
+
+    let num_msg = detector_config.get_num_images() * 4 + 1;
+
+    write_raw_msg_fh(&first_header, &mut file);
+    write_raw_msg_fh(&detector_config_msg, &mut file);
+
+    size += first_header.len();
+    size += detector_config_msg.len();
+
+    let mut buf: Message = Message::new();
+
+    for i in 0..num_msg {
+        loop {
+            match socket.recv(&mut buf, 0) {
+                Ok(_) => break,
+                Err(zmq::Error::EAGAIN) => {
+                    println!("EAGAIN at {num_msg}, offset: {}", num_msg - i);
+                    continue;
+                }
+                Err(err) => {
+                    panic!("{}", err.to_string());
+                }
+            }
+        }
+        write_raw_msg_fh(&buf, &mut file);
+        size += buf.len();
+    }
+
+    println!("Received {} bytes in {:?} s.", size, start.elapsed());
+}
+
 pub fn main() {
     let cli = Cli::parse();
 
+    let env = env_logger::Env::default()
+        .filter_or("LIBERTEM_DECTRIS_LOG_LEVEL", "error")
+        .write_style_or("LIBERTEM_DECTRIS_LOG_STYLE", "always");
+    env_logger::init_from_env(env);
+
     match cli.action {
         Action::Cat { start_idx, end_idx } => action_cat(&cli, start_idx, end_idx),
         Action::Inspect { head, summary } => action_inspect(&cli, head, summary),
         Action::Repeat { repetitions } => action_repeat(&cli, repetitions),
         Action::Sim { uri } => action_sim(&cli.filename, &uri),
+        Action::Record { uri } => action_record(&cli.filename, &uri),
     }
 }
